"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const apollo_server_core_1 = require("apollo-server-core");
const iterall_1 = require("iterall");
function graphqlKoa(options) {
    if (!options) {
        throw new Error('Apollo Server requires options.');
    }
    if (arguments.length > 1) {
        throw new Error(`Apollo Server expects exactly one argument, got ${arguments.length}`);
    }
    const graphqlHandler = (ctx) => {
        return apollo_server_core_1.runHttpQuery([ctx], {
            method: ctx.request.method,
            options: options,
            query: ctx.request.method === 'POST'
                ?
                    ctx.request.body || ctx.req.body
                : ctx.request.query,
            request: apollo_server_core_1.convertNodeHttpToRequest(ctx.req),
            enableDefer: true,
        }).then(({ graphqlResponse, graphqlResponses, responseInit }) => __awaiter(this, void 0, void 0, function* () {
            Object.keys(responseInit.headers).forEach(key => ctx.set(key, responseInit.headers[key]));
            if (graphqlResponse) {
                ctx.body = graphqlResponse;
            }
            else if (graphqlResponses) {
                ctx.set('Content-Type', 'multipart/mixed; boundary="-"');
                const contentTypeHeader = 'Content-Type: application/json\r\n';
                const boundary = '\r\n---\r\n';
                const terminatingBoundary = '\r\n-----\r\n';
                ctx.res.writeHead(200);
                yield iterall_1.forAwaitEach(graphqlResponses, data => {
                    const contentLengthHeader = `Content-Length: ${Buffer.byteLength(data, 'utf8').toString()}\r\n\r\n`;
                    ctx.res.write(boundary + contentTypeHeader + contentLengthHeader + data);
                });
                ctx.res.write(terminatingBoundary);
                ctx.res.end();
            }
        }), (error) => {
            if ('HttpQueryError' !== error.name) {
                throw error;
            }
            if (error.headers) {
                Object.keys(error.headers).forEach(header => {
                    ctx.set(header, error.headers[header]);
                });
            }
            ctx.status = error.statusCode;
            ctx.body = error.message;
        });
    };
    return graphqlHandler;
}
exports.graphqlKoa = graphqlKoa;
//# sourceMappingURL=koaApollo.js.map